@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import random
 
 import numpy as np
@@ -61,6 +62,23 @@ def unique_within_project(project_id, new_labels=[], col_names=["id", "name"]):
     return rets, unique
 
 
+def pre_add_batch(new_labels, se):
+    if connexion.request.headers.get("remove_duplicate_by_name", False):
+        if len(new_labels) == 0:
+            return []
+        curr_labels = Label._get(project_id=new_labels[0].project_id, many=True)
+        curr_names = set(l.name.strip() for l in curr_labels)
+        new_labels_filtered = []
+        for l in new_labels:
+            if l.name.strip() in curr_names:
+                continue
+            curr_names.add(l.name.strip())
+            new_labels_filtered.append(l)
+        return new_labels_filtered
+    else:
+        return new_labels
+
+
 def pre_add(new_label, se):
     # 1. label must have project_id and project must exist,
     if new_label.project_id is None:
@@ -131,7 +149,7 @@ def set_by_project(project_id):
     abort("Not implemented", 500, "Not implemented")
 
 
-get_all, get, post, put, delete = crud(Label, LabelSchema, triggers=[pre_add, pre_delete])
+get_all, get, post, put, delete = crud(Label, LabelSchema, triggers=[pre_add, pre_add_batch, pre_delete])
 
 
 # TODO: abstract to any column
