@@ -1,9 +1,10 @@
+# -*- coding: utf-8 -*-
 from .project import Project
 from .label import Label
 from .task import Task
 from .data import Data
 from .annotation import Annotation
 from .user import User
-from .setting import TaskCategory
+from .setting import TaskCategory, AlembicVersion
 from .tag import Tag
 from .tag_task import TagTask
