@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import json
 
 from paddlelabel.config import db
@@ -11,12 +12,12 @@ class Project(BaseModel):
     __tablename__ = "project"
     __table_args__ = {"comment": "Stores information and settings for each project"}
     project_id = nncol(db.Integer, primary_key=True)
-    name = nncol(db.String(length=256), unique=True)
-    description = db.Column(db.String(length=256))
+    name = nncol(db.String(), unique=True)
+    description = db.Column(db.String())
     task_category_id = db.Column(db.Integer(), db.ForeignKey("taskCategory.task_category_id"))
     task_category = db.relationship("TaskCategory")
-    data_dir = nncol(db.String(length=256), unique=True)
-    label_dir = db.Column(db.String(length=256), unique=True)
+    data_dir = nncol(db.String(), unique=True)
+    label_dir = db.Column(db.String(), unique=True)  # TODO: remove
     labels = db.relationship(
         "Label",
         lazy="selectin",
