@@ -1,12 +1,14 @@
-import os.path as osp
+# -*- coding: utf-8 -*-
+from __future__ import annotations
+from functools import partial
+
+from pathlib import Path
+import os.path as osp  # TODO: remove this dep
 import shutil
-import logging
 
-from paddlelabel.config import db
 from paddlelabel.api import Task
-from paddlelabel.task.util.file import copycontent
-from .util import create_dir, listdir, copy, image_extensions
-from .base import BaseTask
+from paddlelabel.task.util import create_dir, listdir, copy, image_extensions
+from paddlelabel.task.base import BaseTask
 
 
 class Classification(BaseTask):
@@ -20,28 +22,58 @@ class Classification(BaseTask):
             "single_class": self.single_class_exporter,
             "multi_class": self.multi_class_exporter,
         }
-        self.default_importer = self.single_class_importer
+        # TODO: remove this
         self.default_exporter = self.single_class_exporter
 
     def single_class_importer(
         self,
-        data_dir=None,
+        data_dir: Path | None = None,
         filters={"exclude_prefix": ["."], "include_postfix": image_extensions},
     ):
         # 1. set params
-        project = self.project
-        if data_dir is None:
-            data_dir = project.data_dir
-
+        data_dir = Path(self.project.data_dir if data_dir is None else data_dir)
         self.create_warning(data_dir)
-        # 2. import all datas,
-        data_paths = listdir(data_dir, filters)
-        for data_path in data_paths:
-            label_name = osp.basename(osp.dirname(data_path))
-            label = [{"label_name": label_name}] if len(label_name) != 0 else []
-            self.add_task([{"path": data_path}], [label])
 
-        db.session.commit()
+        # 2. import all datas
+        data_paths = [Path(p) for p in listdir(data_dir, filters)]
+        for data_path in data_paths:
+            label_name = data_path.parent.name
+            self.add_task([{"path": str(data_path)}], [[{"label_name": label_name}]])
+
+        self.commit()
+
+    def single_class_exporter(self, export_dir):
+        project = self.project
+        create_dir(export_dir)
+        create_dir(osp.join(export_dir, "no_annotation"))
+        have_no_annotation = False
+
+        # 1. write labels.txt
+        self.export_labels(osp.join(export_dir, "labels.txt"))
+
+        # 2. create label dirs
+        for label in project.labels:
+            create_dir(osp.join(export_dir, label.name))
+
+        # 3. move files to output dir
+        tasks = Task._get(project_id=project.project_id, many=True)
+        new_paths = []
+        for task in tasks:
+            for data in task.datas:
+                label_name = ""
+                if len(data.annotations) == 0:
+                    label_name = "no_annotation"
+                    have_no_annotation = True
+                else:
+                    label_name = data.annotations[0].label.name
+                copy(osp.join(project.data_dir, data.path), osp.join(export_dir, label_name))
+                new_paths.append([osp.join(label_name, osp.basename(data.path))])
+
+        if not have_no_annotation:
+            shutil.rmtree(osp.join(export_dir, "no_annotation"))
+
+        # 4. write split files
+        self.export_split(export_dir, tasks, new_paths)
 
     def multi_class_importer(
         self,
@@ -87,40 +119,7 @@ class Classification(BaseTask):
             labels = labels_dict.get(osp.normpath(data_path), [])
             self.add_task([{"path": data_path}], [[{"label_name": name} for name in labels]])
 
-        db.session.commit()
-
-    def single_class_exporter(self, export_dir):
-        project = self.project
-        create_dir(export_dir)
-        create_dir(osp.join(export_dir, "no_annotation"))
-        have_no_annotation = False
-
-        # 1. write labels.txt
-        labels = self.export_labels(export_dir)
-
-        # 2. create label dirs
-        for label in labels:
-            create_dir(osp.join(export_dir, label.name))
-
-        # 3. move files to output dir
-        tasks = Task._get(project_id=project.project_id, many=True)
-        new_paths = []
-        for task in tasks:
-            for data in task.datas:
-                label_name = ""
-                if len(data.annotations) == 0:
-                    label_name = "no_annotation"
-                    have_no_annotation = True
-                else:
-                    label_name = data.annotations[0].label.name
-                copy(osp.join(project.data_dir, data.path), osp.join(export_dir, label_name))
-                new_paths.append([osp.join(label_name, osp.basename(data.path))])
-
-        if not have_no_annotation:
-            shutil.rmtree(osp.join(export_dir, "no_annotation"))
-
-        # 4. write split files
-        self.export_split(export_dir, tasks, new_paths)
+        self.commit()
 
     def multi_class_exporter(self, export_dir):
         project = self.project
@@ -133,7 +132,7 @@ class Classification(BaseTask):
         have_no_annotation = False
 
         # 2. write labels.txt
-        self.export_labels(export_dir)
+        self.export_labels(osp.join(export_dir, "labels.txt"))
 
         # 3. move all images to image folder
         tasks = Task._get(project_id=project.project_id, many=True)
@@ -156,3 +155,40 @@ class Classification(BaseTask):
 
         # 4. export split
         self.export_split(osp.join(export_dir), tasks, new_paths)
+
+
+class ProjectSubtypeSelector:
+    def __init__(self):
+        self.import_questions = []
+        self.export_questions = []
+        iq = partial(self.add_q, self.import_questions)
+        eq = partial(self.add_q, self.export_questions)
+        iq(
+            "clasSubCags",
+            True,
+            "choice",
+            ["singleClass", "multiClass"],
+            None,
+            None,
+        )
+
+    def add_q(
+        self,
+        question_set: list,
+        question: str,
+        required: bool,
+        type: str,
+        choices: list[str],
+        tips: str | None,
+        show_after: tuple[str, str] | None,
+    ):
+        question_set.append(
+            {
+                "question": question,
+                "required": required,
+                "type": type,
+                "choices": choices,
+                "tips": tips,
+                # "show_after"
+            }
+        )
