@@ -1,6 +1,9 @@
+# -*- coding: utf-8 -*-
+from __future__ import annotations
 import os
 import os.path as osp
 import shutil
+from pathlib import Path
 
 
 # TODO: switch to pathlib
@@ -8,6 +11,58 @@ import shutil
 image_extensions = [".bmp", ".jpg", ".jpeg", ".png", ".gif", ".webp"]
 
 
+def break_path(path: str) -> list[str]:
+    path = path.replace("\\\\", "/")
+    path = path.replace("\\", "/")
+    return path.split("/")
+
+
+def match_by_base_name(data_path, ann_paths, allow_empty=True, allow_multiple=False):
+    data_path = Path(data_path)
+    ann_paths = [Path(p) for p in ann_paths]
+    base_name = data_path.name.split(".")[0]
+    ann_path = filter(lambda p: p.name.split(".")[0] == base_name, ann_paths)
+    ann_path = list(ann_path)
+    if not allow_multiple and len(ann_path) > 1:
+        ann_path = [str(p) for p in ann_path]
+        raise RuntimeError(f"Multiple annotation files {','.join(ann_path)} matche image file {str(data_path)}")
+    if not allow_empty and len(ann_path) == 0:
+        raise RuntimeError(f"No annotation file matches image file {str(data_path)}")
+    return ann_path
+
+
+def ensure_unique_base_name(paths):
+    """Ensure all paths have unique base name, will change file name on disk
+
+    Args:
+        paths (str): Full paths
+    """
+
+    all_names = set()
+    curr_names = set()
+    changed_list = []
+    paths = [Path(p) for p in paths]
+
+    for path in paths:
+        all_names.add(path.name.split(".")[0])
+
+    for path in paths:
+        basename = path.name.split(".")[0]
+        if basename in curr_names:  # duplicate
+            idx = 1
+            while f"{basename}-{idx}" in all_names:
+                idx += 1
+            new_path = path.parent / Path(f"{basename}-{idx}{path.name[len(basename):]}")
+            path.rename(new_path)
+            all_names.add(new_path.name)
+            curr_names.add(new_path.name)
+            changed_list.append((path, new_path))
+        else:
+            curr_names.add(basename)
+
+    return changed_list
+
+
 def create_dir(path):
     if path is None:
         raise RuntimeError("Path to create is None")
@@ -22,7 +77,7 @@ def remove_dir(path):
     shutil.rmtree(path)
 
 
-def listdir(folder, filters={"exclude_prefix": ["."]}):
+def listdir(folder, filters={"exclude_prefix": ["."]}, exact_match_one_of: str | None = None):
     """
     return relative path of all files satisfying filters under the folder and its subfolders
 
@@ -31,7 +86,7 @@ def listdir(folder, filters={"exclude_prefix": ["."]}):
         filters (dict, optional): Four lists, include/exclude_prefix/postfix. Include first, satisfying either include, then exclude fail either one gets excluded.
 
     Returns:
-        list: File paths relative to folder, sorted
+        list: File paths relative to folder
     """
 
     files = []
@@ -42,6 +97,9 @@ def listdir(folder, filters={"exclude_prefix": ["."]}):
             # files.append(osp.normpath(osp.join(root, f)))
             files.append(osp.join(root, f))
     files = [osp.relpath(f, folder) for f in files]
+    if exact_match_one_of is not None:
+        return list(filter(lambda p: osp.basename(p) in exact_match_one_of, files))
+
     # TODO: support regx
     include_prefix = filters.get("include_prefix", [])
     include_postfix = filters.get("include_postfix", [])
@@ -73,7 +131,7 @@ def listdir(folder, filters={"exclude_prefix": ["."]}):
         return True
 
     files = list(filter(exclude, files))
-    files.sort()
+    # files.sort(key=osp.basename)
     # files = [osp.normpath(p) for p in files]
     return files
 
@@ -88,7 +146,7 @@ def copy(src, dst, make_dir=False):
     shutil.copy(src, dst)
 
 
-def copycontent(src, dst):
+def copy_content(src, dst):
     """
     Recursively copy everything in src to dst. Create dst if not exist.
 
@@ -115,7 +173,11 @@ def copycontent(src, dst):
             copy(fsrc, fdst)
 
 
-def expand_home(path):
-    if path[0] == "~":
-        path = osp.join(osp.expanduser("~"), path[2:])
-    return path
+def expand_home(path: Path | str) -> Path:
+    path = str(Path(path).expanduser())
+    return str(path)  # TODO: change to path obj
+
+
+def get_fname(path: str):
+    """get file name from a windows or linux path"""
+    return path.split("\\")[-1].split("/")[-1]
