@@ -1,12 +1,18 @@
+# -*- coding: utf-8 -*-
 import string
 import random
+import platform
+import subprocess
+import sys
+import traceback
+import logging
 
 import connexion
 
 
 def rand_string(length):
     chars = string.ascii_letters + string.punctuation
-    return "".join(random.choice(chars) for x in range(20))
+    return "".join(random.choice(chars) for x in range(length))
 
 
 def camel2snake(string):
@@ -24,6 +30,21 @@ def camel2snake(string):
     return new_string
 
 
+def pyVerGt(version: str = "3.9.0") -> bool:
+    pyVer = list(map(int, platform.python_version().split(".")))
+    recVer = list(map(int, version.split(".")))
+    return pyVer[1] >= recVer[1]
+
+
+def portInUse(port: int) -> bool:
+    import socket
+
+    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    res = sock.connect_ex(("localhost", int(port)))
+    sock.close()
+    return res == 0
+
+
 class Resolver(connexion.resolver.RestyResolver):
     def resolve_operation_id_using_rest_semantics(self, operation):
         """
@@ -64,7 +85,6 @@ class Resolver(connexion.resolver.RestyResolver):
 
             return self.collection_endpoint_name if is_collection_endpoint else method.lower()
 
-        # print(f"{get_controller_name()}.{get_function_name()}")
         return f"{get_controller_name()}.{get_function_name()}"
 
     # TODO: find a better way to resolve this
@@ -72,7 +92,9 @@ class Resolver(connexion.resolver.RestyResolver):
         # TODO: auto resolve /collection/{item}/collection
 
         special = {
+            "/projects/importOptions/{project_type} getImportOptions": "paddlelabel.api.controller.project.import_options",
             "/projects/{project_id}/tasks getTasks": "paddlelabel.api.controller.task.get_by_project",
+            "/projects/{project_id}/tasks setAll": "paddlelabel.api.controller.task.set_all_by_project",
             "/projects/{project_id}/labels getLabels": "paddlelabel.api.controller.label.get_by_project",
             "/projects/{project_id}/labels setLabels": "paddlelabel.api.controller.label.set_by_project",
             "/projects/{project_id}/labels removeLabels": "paddlelabel.api.controller.label.delete_by_project",
@@ -81,11 +103,13 @@ class Resolver(connexion.resolver.RestyResolver):
             "/projects/{project_id}/progress getProgress": "paddlelabel.api.controller.task.get_stat_by_project",
             "/projects/{project_id}/split splitDataset": "paddlelabel.api.controller.project.split_dataset",
             "/projects/{project_id}/export exportDataset": "paddlelabel.api.controller.project.export_dataset",
-            "/projects/{project_id}/import importDataset": "paddlelabel.api.controller.project.import_dataset",
+            "/projects/{project_id}/import importDataset": "paddlelabel.api.controller.project.import_additional_data",
             "/projects/{project_id}/predict predict": "paddlelabel.api.controller.project.predict",
+            "/projects/{project_id}/toEasydata toEasydata": "paddlelabel.api.controller.project.to_easydata",
             "/tasks/{task_id}/tags getTags": "paddlelabel.api.controller.tag.get_by_task",
             "/tasks/{task_id}/tags addTag": "paddlelabel.api.controller.tag.add_to_task",
             "/datas/{data_id}/image getImage": "paddlelabel.api.controller.data.get_image",
+            "/datas/{data_id}/mask getMask": "paddlelabel.api.controller.data.get_mask",
             "/tasks/{task_id}/annotations getAnnotations": "paddlelabel.api.controller.annotation.get_by_task",
             "/tasks/{task_id}/datas getDatas": "paddlelabel.api.controller.data.get_by_task",
             "/datas/{data_id}/annotations getAnnotations": "paddlelabel.api.controller.annotation.get_by_data",
@@ -94,11 +118,13 @@ class Resolver(connexion.resolver.RestyResolver):
             "/rpc/folders getFolders": "paddlelabel.api.rpc.file.get_folders",
             "/rpc/seg/polygon2points polygon2points": "paddlelabel.api.rpc.seg.polygon2points_str",
             "/rpc/seg/points2polygon points2polygon": "paddlelabel.api.rpc.seg.points2polygon_str",
+            "/rpc/cache createCache": "paddlelabel.api.rpc.cache.create_cache",
+            "/rpc/cache/{cache_id} getCache": "paddlelabel.api.rpc.cache.get_cache",
+            "/debug/printid/{debug_id} printDebugId": "paddlelabel.api.rpc.debug.cmdOutputDebugId",
             "/version getVersion": "paddlelabel.api.rpc.monitor.get_version",
             "/samples loadSample": "paddlelabel.api.controller.sample.load_sample",
             "/samples/structure getStructure": "paddlelabel.api.controller.sample.sample_folder_structure",
             "/samples/file getFile": "paddlelabel.api.controller.sample.serve_sample_file",
-            "/datas/{data_id}/mask getMask": "paddlelabel.api.controller.data.get_mask",
         }
         opid = None
 
@@ -119,3 +145,72 @@ class Resolver(connexion.resolver.RestyResolver):
             return f"{router_controller}.{opid}"
 
         return self.resolve_operation_id_using_rest_semantics(operation)
+
+
+def can_update(name: str = "paddlelabel", log: bool = False) -> bool:
+    """
+    Check pypi to see if a package can be updated
+
+    Parameters
+    ----------
+    name : str, optional
+        Package name. Defaults to "paddlelabel"
+    log : bool, optional
+        Whether to print a message to cmd if update is available. Defaults to False
+
+    Returns
+    -------
+    bool
+        If the package is upgradeable from pypi
+    """
+    try:
+        latest_version = str(
+            subprocess.run([sys.executable, "-m", "pip", "install", f"{name}==", "--retries", "1"], capture_output=True)
+        )
+        version_start = "(from versions:"
+        latest_version = latest_version[latest_version.find(version_start) + len(version_start) :]
+        latest_version = latest_version[: latest_version.find(")")]
+        latest_version = latest_version.replace(" ", "").split(",")[-1]
+
+        if latest_version == "none":
+            return False
+
+        current_version = subprocess.run(
+            [sys.executable, "-m", "pip", "show", "{}".format(name)], capture_output=True
+        ).stdout.decode("utf8")
+        if len(current_version) == 0:
+            return False
+        current_version = current_version[current_version.find("Version:") + len("Version:") :]
+        current_version = current_version.split("\n")[0].strip()
+
+        # TODO: zero pad every part of version num @lin
+        if latest_version > current_version:
+            if log:
+                logging.getLogger("paddlelabel").info(
+                    f"""
+Currently installed {name} version is {current_version}, a newer version {latest_version} is available on pypi. Please consider updating {name} with:
+    pip install --upgrade {name}
+You can visit https://github.com/PaddleCV-SIG/PaddleLabel/blob/develop/doc/CN/release_notes.md to see more details about PaddleLabel updates."""
+                )
+            return True
+        else:
+            return False
+    except:
+        logging.getLogger("paddlelabel").exception("Check for update from pypi failed")
+        return False
+
+
+def backend_error(error):
+    """global error handling for backend
+
+    Args:
+        error (Exception): Any exception raised
+
+    Returns:
+        dict, int: response body, status code
+    """
+    print(traceback.format_exc())
+    return {
+        "title": "Backend error: " + str(error),
+        "status": 500,
+    }, 500
